#include <cmath>
#include <math.h>
#include "JetEngineLibrary.h"

// Air properties for engine model initialization

const double k = 1.4;        //specific heat ratio of air at 300K [-]
const double cp = 1005.0;    //specific heat at constant pressure of air at 300 K [J/kg K]
const double R = 287.0;      //specific gas constant of air at 300 K [J/kg K]

namespace JetEngineLibrary
{
	double JetEngineLibrary::JetEngine::GetJetEngineMaxAirFlow(double inlet_diameter, double air_mass_flow_max)
	{
		// Determines air volume flow rate at standard conditions (300K and 101.33 kPa)
		// This requires a one-time iteration to simultaneously solve for inlet velocity
		// and density with specified air mass flow rate and inlet diameter

		const double Pstp = 101330;
		const double Tstp = 300;
		const double pi = atan(1) * 4;
		const double rho_stp = Pstp / (R * Tstp);

		const double inlet_area = 0.25 * pi * pow(inlet_diameter, 2.0);

		double V1stp, T1stp, P1stp;
		double air_volume_flow_max;

		double rho1s = rho_stp;
		int i = 1;

		while (i <= 20)
		{
			air_volume_flow_max = air_mass_flow_max / rho1s;
			V1stp = air_volume_flow_max / inlet_area;
			T1stp = Tstp - pow(V1stp, 2.0) / (2.0*cp);
			P1stp = Pstp / pow((Tstp / T1stp), (k / (k - 1.0)));
			rho1s = P1stp / (R * T1stp);

			i = i + 1;
		}

		return air_volume_flow_max;
	}
	double JetEngine::GetJetEngineThrust(double throttle, bool afterburner, double air_volume_flow_max, double M_0, double T_0, double P_0, double rho)
	{
		double m_fuel, m_ab;
		double Vfr_cor, Vfr_fan;
		double P_1, P_7, P_f;
		double T_1, T_7;
		double V_0, V_1, V_7;
		double c_0, c_7, M_7;
		double P_tf, M_f, T_f, c_f, V_f;
		double P_t0, P_t1, P_t2, P_t3, P_t4, P_t5, P_t6, P_t7;
		double T_t0, T_t1, T_t2, T_t3, T_t4, T_t5, T_t6, T_t7;
		double rho1;
		double m_cor, m_fan;
		double f, f_ab;
		double Q, Q_ab;
		double rp;
		double fan_power;
		double thrust;

		// Stations:

		//	0 - Freestream flight conditions
		//	1 - Diffuser inlet
		//	2 - Compressor inlet
		//	3 - Combustor inlet
		//	4 - Turbine inlet
		//	5 - Afterburner inlet
		//	6 - Nozzle inlet
		//	7 - Nozzle exit
		//  f - Fan exit

		// NOTE: this model implements a simplified ideal Brayton cycle engine analysis, which utilizes a cold air standard where the gas is
		// assumed to be an ideal gas with constant specific heats, which simplifies the calculation but isn't strictly correct. In particular,
		// the engine performance is very sensitive to the value of k (the specific heat ratio). The default value of 1.4 at 300 K tends to significantly
		// overestimate overall engine performance. Adjusting this value lower makes it compare better to air at elevated temperatures and polytropic
		// processes, and so k can be used to "tune" the model to match published performance.

		//Determine fuel and air volume flow rates at current throttle setting

		m_fuel = fuel_mass_flow_max * throttle;       //mass flow rate of fuel at current throttle setting [kg/s]

		if (throttle == 1.0 && afterburner == true)
		{
			m_ab = fuel_mass_flow_max_ab - fuel_mass_flow_max;   //mass flow rate of fuel in afterburner [kg/s]
		}
		else
		{
			m_ab = 0;
		}

		Vfr_cor = (1.0 / (1.0 + bpr)) * air_volume_flow_max * throttle;      //volume flow rate of air through core at current throttle setting [cu. meters/s]
		Vfr_fan = (bpr / (1.0 + bpr)) * air_volume_flow_max * throttle;      //volume flow rate of air through fan at current throttle setting [cu. meters/s]
		
		// Determine velocity at inlet of engine from total volume flow and inlet area

		V_1 = (air_volume_flow_max*throttle) / inlet_area;

		// Determine freestream stagnation temperature and pressure of ambient air based on OAT and Mach number

		T_t0 = T_0 * (1 + 0.5 * (k - 1) * M_0 * M_0);			//stagnation temperature of ambient air
		P_t0 = P_0 * pow((1 + 0.5 * (k - 1) * M_0 * M_0), ((k - 1) / k));	//stagnation pressure of ambient air
		
		// Determine static temperature[K] and pressure[Pa] at inlet of engine

		T_1 = T_t0 - pow(V_1, 2.0) / (2.0*cp);
		P_1 = P_t0 / pow((T_t0 / T_1),(k / (k - 1.0)));

		// Determine air density at inlet of engine [kg / m3]

		rho1 = P_1 / (R * T_1);

		// Determine air mass flow rates through fan and core

		m_cor = Vfr_cor * rho1;  // mass flow rate of air through core at current throttle setting[kg / s]
		m_fan = Vfr_fan * rho1;  // mass flow rate of air through fan at current throttle setting[kg / s]

		//get fuel air ratio in combustor and afterburner
		//If throttle is zero, m_cor is zero which would give div by zero, following prevents that.

		if (throttle == 0)
		{
			f = 0.0;
		}
		else
		{
			f = m_fuel / m_cor;
		}

		//Calculate heat rate into combustor (stochiometric ratio for Jet-A is f ~ 0.067. If f is greater than this, unburned fuel is in products)
	
		if (f <= f_st && f >= 0)
		{
			// complete combustion in core
			Q = m_fuel * HV;
			// fuel air ratio in afterburner = fuel into ab divided by bypass air and remaining air from core.
			f_ab = m_ab / (m_fan + m_cor * (f / f_st));
		}
		else if (f > f_st)   		
		{
			//fuel rich in core, only burned fuel releases heat, no air available for afterburner combustion
			Q = f_st * m_cor * HV;
			// fuel air ratio in afterburner = fuel into ab + unburned fuel from core divided by bypass air.
			f_ab = (m_ab + m_cor * (f - f_st)) / m_fan;
		}

		//Calculate heat rate into afterburner (stochiometric ratio for Jet-A is f ~ 0.067. If f is greater than this, unburned fuel is in products)

		if (f_ab <= f_st && f_ab >= 0)
		{
			// complete combustion in afterburner
			Q_ab = m_ab * HV;
		}
		else if (f_ab > f_st)
		{
			//fuel rich in afterburner, only burned fuel releases heat, only bypass air available for afterburner combustion
			Q_ab = f_st * m_fan * HV;
		}

		// Relate pressure ratio to throttle position (assuming shaft speed proportional to throttle, pressure ratio proportional to speed squared).

		rp = 1.0 + (pow(throttle, 2.0) * (rp_max - 1.0)); //cannot be less than 1.0

		//Calculate fan power assuming isentropic process

		P_f = (rp/rp_max) * fpr * P_1;

		fan_power = Vfr_fan * (P_f - P_1);

		// Determine total/stagnation temperatures at all stations [K]

		T_t1 = T_t0;											//stagnation temperature at diffuser inlet (isentropic, no heat or work exchange)
		T_t2 = T_t1;											//stagnation temperature at compressor inlet (isentropic, no heat or work exchange)
		T_t3 = T_t2 * (pow(rp, ((k - 1) / k)));					//stagnation temperature at compressor outlet (isentropic process)

		if (throttle == 0.0)
		{
			T_t4 = T_t3;
			T_t5 = T_t4;
			T_t6 = T_t5;
		}
		else
		{
			T_t4 = T_t3 + (Q / (m_cor * cp));							//stagnation temperature at turbine inlet
			T_t5 = T_t4 - (T_t3 - T_t2) - (fan_power / (m_cor * cp));   //stagnation temperature at turbine outlet (same work as compressor + fan)
			T_t6 = T_t5 + (Q_ab / (m_cor * cp));						//stagnation temperature at nozzle inlet
		}
		T_t7 = T_t6;											//stagnation temperature in nozzle is constant

		// Determine total/stagnation pressures at all stations [Pa]

		P_t1 = P_t0;					                //stagnation pressure at diffuser inlet (isentropic, no heat or work exchange)
		P_t2 = P_t1;					                //stagnation pressure at compressor inlet (isentropic, no heat or work exchange)
		P_t3 = rp * P_t2;				                //stagnation pressure at compressor outlet (isentropic process)
		P_t4 = P_t3;					                //constant stagnation pressure heat addition in combustor
		P_t5 = P_t4 * pow((T_t5 / T_t4), (k / (k - 1)));//stagnation pressure at turbine outlet
		P_t6 = P_t5;					                //constant stagnation pressure heat addition in afterburner (even when off)
		P_t7 = P_t6;					                //stagnation pressure in nozzle is constant

		// Determine static temperatures and Mach numbers and velocities at stations 0 and 7 needed to calculate core thrust

		c_0 = sqrt(k * R * T_0);	                            //speed of sound in ambient air [m/s]
		V_0 = M_0 * c_0;			                            //velocity relative to ambient air [m/s]

		P_7 = P_0;												//static pressure at exit same as ambient
		M_7 = sqrt(pow((2 / (k + 1)) * (P_t7 / P_7), ((k - 1) / k)));	//Mach number at nozzle exit [-]
		T_7 = (T_t7 / (1 + 0.5 * (k - 1) * M_7 * M_7));		        //static temperature at nozzle exit [K]
		c_7 = sqrt(k * R * T_7);				                //speed of sound at nozzle exit [m/s]
		V_7 = M_7 * c_7;					                    //exit air velocity [m/s]

		// Determine fan exit velocity to calculate fan thrust

		P_tf = fpr * P_t2;                                //Stagnation pressure at exit of fan
		M_f = sqrt(pow((2 / (k + 1)) * (P_tf / P_0), ((k - 1) / k)));	//Mach number at fan exit [-]
		T_f = (T_t2 / (1 + 0.5 * (k - 1) * M_f * M_f));		        //static temperature at fan exit [K]
		c_f = sqrt(k * R * T_f);				                //speed of sound at fan exit [m/s]
		V_f = M_f * c_f;					                    //fan exit air velocity [m/s]

		// Calculate thrust

		thrust = ((m_cor + m_fuel + m_ab) * V_7) - (m_cor * V_0) + (m_fan * (V_f - V_0)); //engine thrust [N]

		return thrust;
	}

	double JetEngine::GetJetEngineIsp(double throttle, bool afterburner, double thrust)
	{
		double m_fuel;
		double Isp_main;

		//Determine fuel flow rate at current throttle setting

		m_fuel = fuel_mass_flow_max * throttle;       //mass flow rate of fuel at current throttle setting [kg/s]

		// Calculate Isp

		if (m_fuel > 0)
		{
			Isp_main = fmin(thrust, max_thrust*1000) / m_fuel;
		}
		else
		{ 
			Isp_main = INFINITY;
		}

		return Isp_main;
	}

	double JetEngine::GetJetEngineAfterburnerIsp(double throttle, bool afterburner, double thrust)
	{
		double m_ab;
		double Isp_ab;

		//Determine fuel flow rates at current throttle setting

		if (throttle == 1.0 && afterburner == true)
		{
			m_ab = fuel_mass_flow_max_ab - fuel_mass_flow_max;   //mass flow rate of fuel in afterburner [kg/s]
			Isp_ab = (thrust - max_thrust*1000) / m_ab;
		}
		else
		{
			m_ab = 0;
			Isp_ab = INFINITY;
		}

		return Isp_ab;
	}
}

