// JetEngineLibrary.h
// by Thunder Chicken

#pragma once
#include <cmath>

// Minimum engine parameters that must be provided for model at 100% throttle at sea level.
// Note that maximum thrust is needed to calculate specific impulse (Isp) from 
// specific fuel consumption (sfc), and is used to set the maximum thrust level of the thrusters
// in Orbiter, but is not otherwise used to calculate thrust. However, the maximum
// thrust calculated by the model should be close to these input values.

// Parameters for SNECMA-M53-P2 Turbofan

const double air_mass_flow_max = 92.0;			//maximum air flow rate through engine [kg/s]
const double inlet_diameter = 0.796;			//inlet diameter of engine (fan diameter if turbofan) [m]
const double max_thrust = 65.0;                 //maximum thrust without afterburner [kN]
const double rp_max = 9.8;						//compressor pressure ratio at full throttle [-]
const double sfc = 32.4;                        //specific fuel consumption at full throttle [g/kN s]
const double bpr = 0.36;                        //turbofan bypass ratio (mass flow through fan / mass flow through core, 0 for pure turbojets)
const double fpr = 2.0;							//turbofan pressure ratio (0 for pure turbojets)

const double max_thrust_ab = 95.0;              //maximum thrust with afterburner [kN]
const double sfc_ab = 53.0;                     //specific fuel consumption with afterburner [g/kN s]

// Fuel properties (Jet-A)

const double HV = 42.8e+06;  //lower heating value of jet-A [J/kg]
const double f_st = 0.067;   //stochiometric fuel air ratio for jet-A [-]

// Calculate inlet area of engine from inlet diameter

const double pi = atan(1) * 4;
const double inlet_area = 0.25 * pi * pow(inlet_diameter, 2.0);

// Maximum Isp of engine from SFC definition and fuel flow rates

const double Isp_max = 1e06 / sfc;				//Isp of engine without afterburning [N-s/kg]
const double Isp_max_ab = 1e06 / sfc_ab;		//Isp of engine with afterburning [N-s/kg]

const double fuel_mass_flow_max = sfc * max_thrust * 1e-03;				//total fuel flow rate at maximum thrust without afterburner [kg/s]
const double fuel_mass_flow_max_ab = sfc_ab * max_thrust_ab * 1e-03;	//total fuel flow rate at maximum thrust with afterburner [kg/s]

namespace JetEngineLibrary
{
	class JetEngine
	{
	public:
		
		static double GetJetEngineMaxAirFlow(double inlet_diameter, double air_mass_flow_max);
		static double GetJetEngineThrust(double throttle, bool afterburner, double air_volume_flow_max, double M_0, double T_0, double P_0, double rho);
		static double GetJetEngineIsp(double throttle, bool afterburner, double thrust);
		static double GetJetEngineAfterburnerIsp(double throttle, bool afterburner, double thrust);

	};
}